local Matrix = {}

function Matrix:new(r, c, matrix_type, default)
	local matrix_type = matrix_type or "double"
	local m = {n_rows=r, n_cols=c, value_type=matrix_type}
	for i=1,r do
		m[i] = {}
		for j=1,c do
			m[i][j] = default
		end
	end
	setmetatable(m, self)
	self.__index = self
	return m
end

function Matrix:new_from_initializer(r, c, matrix_type, initializer)
	local matrix_type = matrix_type or "double"
	local m = {n_rows=r, n_cols=c, value_type=matrix_type}
	for i=1,r do
		m[i] = {}
		for j=1,c do
			m[i][j] = initializer(i,j)
		end
	end
	setmetatable(m, self)
	self.__index = self
	return m
end

function Matrix:copy(i, j, n, m)
	local new = Matrix:new(n, m, self.value_type)
	for x=1,n do
		for y=1,m do
			new[x][y] = self[i+x-1][j+y-1]
		end
	end
	return new
end

function Matrix:print()
	for i=1,self.n_rows do
		io.write("R", i, ": \t")
		for j=1,self.n_cols do
			io.write(self[i][j], "\t")
		end
		io.write("\n")
	end
end

return Matrix
--[[
t = Matrix:new_from_initializer(20, 20, "int", function(i, j) return i end)
a = t:copy(5,5,3,3)
a:print()
--]]